title: Reclaiming the Computing Commons
url: https://www.jacobinmag.com/2016/02/free-software-movement-richard-stallman-linux-open-source-enclosure/
hash_url: e6f57bb2a95da1508f0bb905bc08471c

<p><span class="dropcaps">C</span>apital takes. The commodification and exploitation of nature; the enclosure of the intellectual and informational commons in medicine, agriculture, and other areas of technical knowledge; the expropriation of public space to secure profit — all define an economic system supposedly premised on freedom.</p>
<p>Could the world of computing offer an alternative vision? Could it even aid in arresting enclosure’s march?</p>
<p><a href="http://www.gnu.org/philosophy/free-sw.html">Software freedom</a> — the core commitment of the free software movement — does represent at least the rudiments of a better system. Resisting and reversing enclosure will not come about through “sustainable growth” or the “sharing economy,” which preserve the logics and structures of the status quo. “Openness,” or the conviction that norms of transparency and publicity will clarify (and thereby equalize) power relations, is also <a href="http://www.publicbooks.org/nonfiction/open-markets-open-projects-wikipedia-and-the-politics-of-openness">no solution at all</a>.</p>
<p>Resisting enclosure requires a more radical vision in which the productive use of the commons is managed and preserved through conscious planning and collective effort. The free software movement is an example of such resistance.</p>
<h4>Free Software vs. Open Source</h4>
<p><span class="dropcaps2">T</span>o understand what is meant by software freedom — which is predicated on a commitment to preserving an information commons — it is necessary to explore the distinctions between free software and open source software.</p>
<p>Open source software is common and widespread. Millions of people use open source code every day, from popular web browsers to the software kernels powering Kindles to suites of applications that are free alternatives to expensive proprietary software like Microsoft Office. Such software is “open” in the sense that it is not licensed under terms that prohibit inspecting or modifying its underlying code.</p>
<p>Open source is in fact a derivative and a departure from an earlier model: <a href="http://www.gnu.org/philosophy/free-sw.en.html">free software</a>, or software that users and distributors are able to use, modify, and distribute as they see fit.</p>
<p>Developers secure this freedom by restricting attempts to enclose the free software commons. “Copyleft” licenses like the <a href="http://www.gnu.org/licenses/gpl-3.0.en.html">GNU Public License</a> are grafted onto the existing structure of copyright and software licensing to ensure that if free software is modified, it cannot then be withdrawn from the commons and distributed as if it were a commodity. Users who accept the terms of a copyleft license commit to perpetuating free access to the licensed code. The freedom of free software is secured by the restrictions that copyleft places on an individual’s ability to withdraw code from the commons.</p>
<p>Like <a href="http://www.gnu.org/philosophy/free-sw.html">free software</a>, <a href="https://opensource.org/osd-annotated">open source</a> software is typically packaged with its associated source code under a copyleft license. However, some non-copyleft licenses vary in the degree to which they require those altering the licensed software to distribute its source code.</p>
<p>“Permissive” licenses do not obligate those who redistribute software to release or make available its source code. As such, permissive licensure cannot guarantee that future iterations of code will always be publicly available. While these licenses are still technically free, their use by some prominent open source projects points to the key dividing line between free software and open source: the differing political inclinations of the movements behind them, rather than the shared characteristics of their distribution.</p>
<p>Many commercial firms make software available through permissive open source licenses but seek to restrict how modifications to their software are distributed, or insist on retaining a wider set of intellectual property rights in their software than restrictive copyleft licenses would allow.</p>
<p>Open source is also distinguishable from free software in terms of the arguments proponents make to individual users. While free software is presented as marching under the banner of freedom, open source often relies on justifications of possession and control. According to the advocates of open source, users who own their own devices should retain control over the code executed on them, rather than relinquishing that authority by running programs whose internals are (legally, if not always technically) opaque.</p>
<p>The primary aim of free software — which is based on freely available code — is not to empower entrepreneurs and exploit coders and hackers; it is to enable individuals who use computing power for their own ends to employ and alter the code on their computers in any way they see fit. Open source, by contrast, is not necessarily incompatible with the imperatives of capital. A variety of firms pursue profits through the sale of services or support related to open source software.</p>
<p>Yet it is also possible to overstate the case for free software.</p>
<p>It is certainly not an emancipatory political movement. Individual freedom to use and modify code is hardly a sufficient condition for waging radical political struggle; it may not even be a necessary condition. Increased interconnectivity and advances in computing power <a href="http://www.hup.harvard.edu/catalog.php?isbn=9780674032927">do not automatically advance</a> emancipatory or socially valuable aims.</p>
<p>Whether software is free, code — along with the physical infrastructure it’s running on — is always subject to control, whether individual or social. The vital political question in software development, then, is not, “What are the restrictions on individual modification of software?” but rather, “Who controls the processes of computing?”</p>
<p>In other words, free software isn’t socialism for your computer. It can furnish useful tools and models, but the broader project of reclaiming the computing commons requires the articulation of a political agenda and the mobilization necessary to pursue it.</p>
<h4>The Social and the Political</h4>
<p><span class="dropcaps2">N</span>ot only is the software freedom movement not an emancipatory political movement, it probably can’t be described as a political movement at all. We shouldn’t confuse consumption habits — such as the choice to use free software, or to abstain from consuming closed source software — for political engagement. Consumption is not a politically combative act — refraining from consumption even less so.</p>
<p>To be sure, the struggles against the closure of the information commons and the commodification of socially produced information — as part of the broader struggle between exploiters and exploited — are by nature political. But free software development is not an autonomous site of production that is disconnected from the market.</p>
<p>Coding skills are honed and maintained in academic and industrial contexts where proprietary code is used, developed, and marketed. Requesting payment for free software’s development is not, in itself, a violation of the free software ethic, which requires only that people not be prohibited from altering or redistributing code. But the free software movement has not, and indeed could not, serve as the template for a profit-driven software development industry founded on the exploitation of coders’ labor.</p>
<p>As such, the social conditions under which free software is developed are marked by a contradiction between hackers’ commitments to the promotion of software freedom and their dependence on the market to reproduce themselves, socially and professionally. Free software depends socially, if not always computationally, on proprietary software development.</p>
<p>It is difficult to conceive of free software hackers — still less users — as artisanal craft workers, producing their code and reproducing themselves in a digital Arcadia of self-sufficiency. It is close to impossible for a single individual to code, compile, and debug all the software running on a modern computer.</p>
<p>Peek at the software repositories under the hood of any <a href="https://en.wikipedia.org/wiki/Linux_distribution">Linux distribution</a>, and you will see an enormous concatenation of labor in the form of thousands of applications and millions of lines of code. Beyond free software’s social dependence on software development more generally (and mirroring copyleft’s reliance on existing legal frameworks) the availability of fully functional and widely available free software rests on the highly coordinated cooperation of large numbers of workers.</p>
<p>Open source advocates frequently point to the technical advantages of openness, while free software advocates retain their commitment to the social (rather than technical) goods that arise from preserving the information commons. But this attention to the social doesn’t always have a political valence. As anthropologist Gabriella Coleman stresses in her invaluable <a href="http://gabriellacoleman.org/Coleman-Coding-Freedom.pdf">study</a> of hackers, free software advocates — in both the free software and open source traditions — are often chary of political arguments.</p>
<p>Instead, Coleman says hackers frequently prefer ethical arguments grounded in the values of cooperation and sharing. This is in tune with contemporary liberalism, where the ethical is elevated above the political, and the frontiers of the possible are marked by the possibility of persuasion rather than the pursuit of power.</p>
<h4>Digital William Morrises</h4>
<p><span class="dropcaps2">T</span>he ethos of software hacking draws inspiration from multiple traditions. Many hackers see themselves as craft workers, or self-identify as hieratic experts in technically advanced disciplines, insulated from the pressures and dangers that beset other workers. According to sociologist Andrew Ross, hackers’ consciousness of themselves as workers <a href="https://books.google.com/books?id=-WMTCgAAQBAJ&amp;pg=PA168&amp;lpg=PA168&amp;dq=#v=onepage&amp;q&amp;f=false">is predicated</a> on the belief that “their expertise will keep them on the upside of the technology curve that protects the best and brightest from proletarianization.”</p>
<p>At their most romantic, free software advocates seem like digital William Morrises: prophets of the impending future of an alternative present, in which the specialized production of valuable artifacts is the consequence of individual care, deep artisanal knowledge, and webs of trust between producers and consumers.</p>
<p>It is appealing to imagine a software regime in which the development community self-consciously preserves an information commons. But this is not the only impulse drawing the hackers’ cart. Sharing the yoke is another, rather different beast: the hyper-individualist (and often masculinist) political vision of so many techno-utopians.</p>
<p>As Ross <a href="https://muse.jhu.edu/login?auth=0&amp;type=summary&amp;url=/journals/american_quarterly/v058/58.3ross.html">notes</a>, within the techno-utopian imaginary, “libertarian concerns about the freedom of consumer choices hold sway to the detriment of attention to labor issues.” Such rhetoric frequently glorifies the image of the omni-competent and self-empowered hacker, and evinces little (if any) concern about actual <a href="http://www.amazon.com/Digital-Labour-Karl-Christian-Fuchs/dp/0415716160/ref=la_B001JSCH8U_1_4?s=books&amp;ie=UTF8&amp;qid=1441972004&amp;sr=1-4">labor</a> <a href="https://viewpointmag.com/2015/09/08/cyber-proletariat-an-interview-with-nick-dyer-witheford/">conditions</a> in the technology sector — especially not the extreme exploitation prevalent in the mining and manufacturing industries that provide software’s physical preconditions.</p>
<p>One of the main obstacles to a more deeply developed political consciousness in the hacker community arises from its contradictory sources of inspiration. Free software development depends on the many small contributions of disparate individuals; but it also depends on the concentration (and valorization) of high levels of expertise among a smaller subset of those individuals. The free software movement is committed to both the choice and freedom of anyone who uses a computer and the cult of the individual — the capable hacker overcoming challenges through technical mastery.</p>
<p>The combination of an egalitarian ethos with a libertarian ideology sometimes manifests itself in a crabbed, contradictory political vision that idealizes “<a href="http://blogs.wsj.com/digits/2015/08/21/silicon-valley-icon-lawrence-lessig-wants-to-hack-his-way-to-the-presidency/">hacking</a>” political institutions rather than engaging or challenging them, and often defaults to an ideology of anti-consumption libertarianism.</p>
<p>When “openness” trumps attentiveness to the social foundations of computing, possibilities for political action risk being eclipsed by a complacent faith in technology.</p>
<p>Faith in technology as a cure for social ills is a form of mystification. As the science fiction writer Joanna Russ <a href="http://www.depauw.edu/sfs/backissues/16/russ16.htm">observed</a> in the late 1970s, technology talk frequently imagines its subject as separate from and somehow outside of social relations: it “becomes a kind of autonomous deity which can promise both salvation and damnation.”</p>
<p>Technology is typically seen as a force or natural law whose developments are impervious to human control — and therefore above social critique. In this way technological determinism often transforms the radical potential of calling attention to the social and political possibilities of software into an aesthetic posture.</p>
<p>There’s nothing wrong with aesthetic enjoyment, and the joys of free software are many and varied. Hackers enjoy tinkering with and improving code; distribution maintainers and debuggers enjoy managing and participating in technically complicated projects; and end-users enjoy the fruits of this collaboration.</p>
<p>But these pleasures do not translate into political outcomes unless they are wedded to explicitly political activities. Free software is not an emancipatory politics, using free software is not a form of political participation, and opting out of the closed software paradigm does not challenge capital’s hegemony over computing.</p>
<p>The struggle against the commodification of the information commons is a political, not technical one.</p>
<h4>Outside the Cash Nexus</h4>
<p><span class="dropcaps2">T</span>hree decades after Richard Stallman’s <a href="http://www.gnu.org/gnu/manifesto.en.html">appeal for software freedom</a>, software development and distribution still remains a powerful source of profits for a handful of mammoth firms. Open source software has overtaken free software in public awareness, and the increasing interconnectivity of our everyday lives is a vector for surplus value extraction and constant surveillance, rather than a harbinger of digital democracy.</p>
<p>Recovering the commons in any productive domain requires class consciousness, organization, and counter-power to take aim at the commodification of social goods. And while free software itself won’t bring us closer to recovering the commons, the gains that free software has made are nevertheless impossible to deny, and offer several important lessons.</p>
<p>The free software movement is more programmatically coherent and ideologically attractive than open source. Free software advocates’ attention to the social conditions (legal frameworks such as copyright, as well as terrains of corporate power and influence) in which software inheres has granted them a vision more sophisticated than “openness” boosters.</p>
<p>Free software also furnishes models for emulation in other fights against the enclosure of the commons and <a href="http://thenewinquiry.com/essays/sowing-scarcity/">the imposition of scarcity</a>. By clamoring for the creation of <a href="https://www.jacobinmag.com/2013/09/property-and-theft/">intellectual property</a> markets in domains ranging from seed stocks to digital goods, capitalists frequently seek to maintain scarcities of goods in order to force people to buy them as commodities.</p>
<p>The consolidation of property in intangible goods as a legal category — which requires the exertion of considerable state power, contrary to the libertarian rhetoric of techno-utopianism — remains powerfully attractive to capitalists looking to secure profits through the commercial distribution of software.</p>
<p>Perhaps most importantly, the free software movement has demonstrated the possibility of building and maintaining a network of relations that can produce, distribute, and refine goods that, if not quite held in common, are at least available through avenues other than the cash nexus.</p>
<p>Over a decade ago Bill Gates insinuated that free software was <a href="http://www.cnet.com/news/gates-taking-a-seat-in-your-den/4/">redolent of communism</a>. He wasn’t wrong. Free software presages something very different from the regime of accumulation under which Microsoft flourishes — a system in which the commons benefit people rather than capital.</p>
